@@ -4,6 +4,7 @@
   Copyright 2001-2007 Wilmer van der Gaast
   Copyright 2007-2008 Y Giridhar Appaji Nag
   Copyright 2016      Stephen Thirlwall
+  Copyright 2017      Ismael Luceno
 
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
@@ -153,7 +154,12 @@ long long int ftp_size( ftp_t *conn, char *file, int maxredir )
 		if( size - i <= 10 )
 		{
 			size *= 2;
-			reply = realloc( reply, size );
+			char *tmp = realloc( reply, size );
+			if (!tmp) {
+				free( reply );
+				return( -1 );
+			}
+			reply = tmp;
 			memset( reply + size / 2, 0, size / 2 );
 		}
 	}
