@@ -6,8 +6,11 @@
   Copyright 2008-2009 Philipp Hagemeister
   Copyright 2015      Joao Eriberto Mota Filho
   Copyright 2016      Ivan Gimenez
+  Copyright 2016      Phillip Berndt
   Copyright 2016      Sjjad Hashemian
   Copyright 2016      Stephen Thirlwall
+  Copyright 2017      David Polverari
+  Copyright 2017      Ismael Luceno
 
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
@@ -46,9 +49,7 @@ int http_connect( http_t *conn, int proto, char *proxy, char *host, int port, ch
 {
 	char base64_encode[64] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
 		"abcdefghijklmnopqrstuvwxyz0123456789+/";
-	char auth[MAX_STRING];
 	conn_t tconn[1];
-	int i;
 
 	strncpy( conn->host, host, MAX_STRING );
 	conn->proto = proto;
@@ -82,9 +83,10 @@ int http_connect( http_t *conn, int proto, char *proxy, char *host, int port, ch
 	}
 	else
 	{
+		char auth[MAX_STRING];
 		memset( auth, 0, MAX_STRING );
 		snprintf( auth, MAX_STRING, "%s:%s", user, pass );
-		for( i = 0; auth[i*3]; i ++ )
+		for( int i = 0; auth[i*3]; i ++ )
 		{
 			conn->auth[i*4] = base64_encode[(auth[i*3]>>2)];
 			conn->auth[i*4+1] = base64_encode[((auth[i*3]&3)<<4)|(auth[i*3+1]>>4)];
@@ -283,25 +285,44 @@ void http_filename( http_t *conn, char *filename )
 	}
 }
 
+inline static char decode_nibble( char n )
+{
+	if( n <= '9' )
+		return( n - '0' );
+	if( n >= 'a' )
+		n -= 'a' - 'A';
+	return( n - 'A' + 10 );
+}
+
+inline static char encode_nibble( char n )
+{
+	return( n > 9 ? n + 'a' - 10 : n + '0' );
+}
+
+inline static void encode_byte( char dst[3], char n )
+{
+	*dst++ = '%';
+	*dst++ = encode_nibble( n >> 4 );
+	*dst = encode_nibble( n & 15 );
+}
+
 /* Decode%20a%20file%20name */
 void http_decode( char *s )
 {
-	char t[MAX_STRING];
-	int i, j, k;
+	for( ; *s && *s != '%'; s ++ );
+	if( !*s )
+		return;
 
-	for( i = j = 0; s[i]; i ++, j ++ )
-	{
-		t[j] = s[i];
-		if( s[i] == '%' )
-			if( sscanf( s + i + 1, "%2x", &k ) )
-			{
-				t[j] = k;
-				i += 2;
-			}
-	}
-	t[j] = 0;
-
-	strcpy( s, t );
+	char *p = s;
+	do {
+		if( !s[1] || !s[2] )
+			break;
+		*p++ = ( decode_nibble( s[1] ) << 4 ) | decode_nibble( s[2] );
+		s += 3;
+		while( *s && *s != '%' )
+			*p ++ = *s ++;
+	} while( *s == '%' );
+	*p = 0;
 }
 
 void http_encode( char *s )
@@ -317,14 +338,14 @@ void http_encode( char *s )
 		}
 
 		t[j] = s[i];
-		if( s[i] == ' ' )
+		if( s[i] <= 0x20 || s[i] >= 0x7f )
 		{
 			/* Fix buffer overflow */
 			if (j >= MAX_STRING - 3) {
 				break;
 			}
 
-			strcpy( t + j, "%20" );
+			encode_byte( t + j, s[i] );
 			j += 2;
 		}
 	}
